AUTHENTICATION_BACKENDS = [ # Needed to login by username in Django admin, regardless of `allauth`
'django.contrib.auth.backends.ModelBackend',

    # `allauth` specific authentication methods, such as login by e-mail
    'allauth.account.auth_backends.AuthenticationBackend',

]

ACCOUNTS_APPS = [
    'allauth',
    'allauth.account',
    'dj_rest_auth.registration',
    ########## Auth Providers ##########
    'allauth.socialaccount',
    'allauth.socialaccount.providers.google',
    'allauth.socialaccount.providers.microsoft',
    "rest_framework",
    'rest_framework.authtoken',
]

REST_FRAMEWORK = {
    "NON_FIELD_ERRORS_KEY": "error",
    "DEFAULT_RENDERER_CLASSES": ("rest_framework.renderers.JSONRenderer",),
    "DEFAULT_PERMISSION_CLASSES": ("rest_framework.permissions.IsAuthenticated",),
    "DEFAULT_AUTHENTICATION_CLASSES": (
        # "rest_framework.authentication.BasicAuthentication",
        # "rest_framework.authentication.SessionAuthentication",
        'rest_framework_simplejwt.authentication.JWTAuthentication', 

    ),
    # "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.LimitOffsetPagination",
    # "PAGE_SIZE": 50,
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
}

AUTH_USER_MODEL = "accounts.User"

SITE_ID = 1
ACCOUNT_USER_MODEL_USERNAME_FIELD = None
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_USERNAME_REQUIRED = False
ACCOUNT_AUTHENTICATION_METHOD = 'email'
ACCOUNT_ADAPTER= 'accounts.adapter.MyCustomAdapter'
SOCIALACCOUNT_ADAPTER= 'accounts.adapter.MyCustomSocialAdapter'
REST_USE_JWT = True
JWT_AUTH_COOKIE = 'jwt-auth'
USER_FIRST=True
USER_NOPASSWORD=True
USER_DELETE=False


from django.utils.translation import gettext_lazy as _
from allauth.account.adapter import DefaultAccountAdapter
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter
from .models import User
from allauth.socialaccount.models import EmailAddress
from django.core.exceptions import ValidationError
from django.conf import settings


class MyCustomAdapter(DefaultAccountAdapter):
    custom_error_messages= {
         "not_active": _(
            "User is not active!"
        ),
    }
    error_messages = dict(list(DefaultAccountAdapter.error_messages.items()) + list(custom_error_messages.items()))
 

    def respond_user_inactive(self, request, user):
        raise ValidationError(self.error_messages["not_active"])


class MyCustomSocialAdapter(DefaultSocialAccountAdapter):
    if settings.USER_FIRST:

        custom_error_messages= {
            "no_user": _(
                "User Does not exist!"
            ),
        }
        error_messages = dict(list(DefaultAccountAdapter.error_messages.items()) + list(custom_error_messages.items()))
    

        def pre_social_login(self, request, sociallogin):
            social_email=sociallogin.email_addresses[0].email

            # social account already exists, so this is just a login
            if sociallogin.is_existing:
                existing_user = User.objects.get(email__iexact=social_email)
                existing_user.first_name=sociallogin.user.first_name
                existing_user.last_name=sociallogin.user.last_name
                existing_user.save()
                return

            # some social logins don't have an email address
            if not sociallogin.email_addresses:
                return
                
            try:
                existing_user = User.objects.get(email__iexact=social_email)
                existing_user.first_name=sociallogin.user.first_name
                existing_user.last_name=sociallogin.user.last_name
                existing_user.save()
            except User.DoesNotExist:
                raise ValidationError(self.error_messages["no_user"])

            # Creat email Addresses 
            if  sociallogin.email_addresses:
                try:
                    new_email=EmailAddress(
                        user=existing_user,
                        email=social_email,
                        verified=sociallogin.account.extra_data['verified_email'],
                        primary=True
                    )
                    new_email.save()
                except:
                 new_email=EmailAddress(
                    user=existing_user,
                    email=social_email,
                    verified=True,
                    primary=True
                )
                new_email.save()
            # Connect existing user to the logined social account
            sociallogin.connect(request, existing_user)
    else:
        pass



import subprocess
import sys
from django.core.management.base import BaseCommand

packages = ['django-allauth','dj_rest_auth','djangorestframework-simplejwt','djangorestframework']
    
class Command(BaseCommand):
    """
    Install packages django-allauth, dj-rest-auth, djangorestframework-simplejwt, djangorestframework
    """

    def _init_pkg(self):
        for package in packages:
            subprocess.check_call([sys.executable, "-m", "pipenv", "install", package])

    def handle(self, *args, **options):
        self._init_pkg()


from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)
from django.db import models

class AbstractAuditModel(models.Model):
    # A timestamp representing when this object was created.
    createdDate = models.DateTimeField(auto_now_add=True)

    # A timestamp reprensenting when this object was last updated.
    updatedDate = models.DateTimeField(auto_now=True)

    # A field to know who updated the record.
    updatedBy =  models.CharField(max_length=200, null=True, blank=True)

    # A field to know who created the record.
    createdBy = models.CharField(max_length=200, default="System")

    # A field to know who deleted the record.
    deletedBy =  models.CharField(max_length=200, null=True, blank=True)

    # A timestamp reprensenting when this object was deleted.
    deletedDate = models.DateTimeField(null=True, blank=True)

    class Meta:
        abstract = True
        # ordering = ["-date_created", "-date_updated"]


class UserManager(BaseUserManager):
    """
    Django requires that custom users define their own Manager class. By
    inheriting from `BaseUserManager`, we get a lot of the same code used by
    Django to create a `User` for free.

    All we have to do is override the `create_user` function which we will use
    to create `User` objects.
    """

    def create_user(self, email, password=None, **extra_fields):
        """
        Create and return a `User` with an email and password.
        """
        if email is None:
            raise TypeError("Users must have an email address.")

        user = self.model(email=self.normalize_email(email), **extra_fields)
        user.set_password(password)
        user.save()

        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Create and return a `User` with superuser powers.
        """
        if password is None:
            raise TypeError("Superusers must have a password.")

        user = self.create_user(email, password, **extra_fields)
        user.is_superuser = True
        user.is_staff = True
        user.is_active = True
        user.save()

        return user


class User(AbstractBaseUser, PermissionsMixin, AbstractAuditModel):
    email = models.EmailField(db_index=True, unique=True)
    first_name = models.CharField(max_length=30, blank=True)
    last_name = models.CharField(max_length=30, blank=True)

    is_active = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)
    REQUIRED_FIELDS=[]
    USERNAME_FIELD = "email"

    objects = UserManager()

    def __str__(self):
        return self.email

    def get_full_name(self):
        return f"{self.first_name} {self.last_name}"

    def get_short_name(self):
        if not self.first_name:
            return self.email
        return self.first_name



from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.authentication import SessionAuthentication, BasicAuthentication, TokenAuthentication
from allauth.socialaccount.providers.google.views import GoogleOAuth2Adapter
from allauth.socialaccount.providers.microsoft.views import  MicrosoftGraphOAuth2Adapter
from allauth.socialaccount.providers.oauth2.client import OAuth2Client
from dj_rest_auth.registration.views import SocialLoginView
from .serializers import UserSerializer



class UserAPIView(GenericAPIView):
    """Return basic info of the currently logged user"""

    serializer_class = UserSerializer
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        if request.user:
            serializer = self.serializer_class(request.user)
            return Response(serializer.data, status=status.HTTP_200_OK)
        return Response({}, status=status.HTTP_400_BAD_REQUEST)

class GoogleLogin(SocialLoginView): # if you want to use Authorization Code Grant, use this

    adapter_class = GoogleOAuth2Adapter
    # callback_url = "http://localhost:8000/accounts/google/login/callback/"
    client_class = OAuth2Client
  


class MSLogin(SocialLoginView): # if you want to use Authorization Code Grant, use this
    
    adapter_class = MicrosoftGraphOAuth2Adapter
    callback_url = "http://localhost:8000/accounts/ms/login/callback/"
    client_class = OAuth2Client

   
class Secure_api(APIView):
    authentication_classes = [SessionAuthentication, BasicAuthentication, TokenAuthentication]
    permission_classes = [IsAuthenticated]
       
    def get(self, request, format=None):
        content = {
            'user': str(request.user),  # `django.contrib.auth.User` instance.
            'auth': str(request.auth),  # None
        }
        return Response(content)




from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.utils.translation import gettext_lazy as _
from .models import User
from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.conf import settings

if settings.USER_NOPASSWORD:
    class UserCreationForm(UserCreationForm):
        """
        A UserCreationForm with optional password inputs.
        """

        def __init__(self, *args, **kwargs):
            super(UserCreationForm, self).__init__(*args, **kwargs)
            self.fields['password1'].required = False
            self.fields['password2'].required = False
            # If one field gets autocompleted but not the other, our 'neither
            # password or both password' validation will be triggered.
            self.fields['password1'].widget.attrs['autocomplete'] = 'off'
            self.fields['password2'].widget.attrs['autocomplete'] = 'off'

        def clean_password2(self):
            password1 = self.cleaned_data.get("password1")
            password2 = super(UserCreationForm, self).clean_password2()
            if bool(password1) ^ bool(password2):
                raise forms.ValidationError("Fill out both fields")
            return password2

@admin.register(User)
class UserAdmin(BaseUserAdmin):
    if settings.USER_NOPASSWORD:
        add_form= UserCreationForm
        add_fieldsets = (
        (None, {"classes": ("wide",), "fields": ("email","is_active")}),
    )
    else:
        add_fieldsets = (
            (None, {"classes": ("wide",), "fields": ("email","password1","password2",)}),
        )
    readonly_fields = ("last_login", "updatedDate", "createdDate","deletedDate")
    fieldsets = (
        (None, {"fields": ("email", "password")}),
        (_("Personal info"), {"fields": ("first_name", "last_name")}),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                )
            },
        ),
        (
            _("Important dates"),
            {
                "fields": (
                    "last_login",
                    "updatedDate",
                    "createdDate",
                )
            },
        ),
    )
    list_display = (
        "email",
        "is_staff",
        "is_active",
    )
    list_filter = (
        "is_staff",
        "is_superuser",
        "is_active",
        "groups",
       "createdDate",
        "updatedDate",
    )
    search_fields = ("id", "email")
    ordering = ("createdDate",)
    date_hierarchy = "updatedDate"
    filter_horizontal = ("groups", "user_permissions")
    actions=['activate_user', 'deactivate_user']
    exclude = ('password',)
    
    def has_delete_permission(self, request, obj=None) :
        return settings.USER_DELETE
        
  
    @admin.action(description='Activate User')
    def activate_user(self, request, queryset):
        user = queryset.update(is_active=True)
        self.message_user(
            request,
            f'{user} User has been activated!'
        )

    @admin.action(description='Deactivate User')
    def deactivate_user(self, request, queryset):
        user = queryset.update(is_active=False)
        self.message_user(
            request,
            f'{user} User has been deactivated!'
        )



from rest_framework import serializers

from .models import User


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ("email", "first_name", "last_name", "is_active")



import email
from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand
from django.db import transaction
from os import environ

USERS = [
    {
        "email": environ["SUPER_USERNAME"],
        "password": environ["SUPER_PASSWORD"],
        "is_staff": True,
        "is_active": True,
        "is_superuser": True,
    },
]


User = get_user_model()


class Command(BaseCommand):
    """
    Initialize users
    """

    @transaction.atomic
    def _init_users(self):
        for user in USERS:
            if User.objects.filter(email=user["email"]).exists():
               self.stdout.write(self.style.WARNING("Users already initialized"))
               return
            password = user.pop("password")
            newuser = User.objects.create(**user)
            newuser.set_password(password)
            newuser.save()
            self.stdout.write(self.style.SUCCESS("%d users inserted" % len(USERS)))

    def handle(self, *args, **options):
        self._init_users()



from django.urls import path
from .views import *
urlpatterns = [
    path("user/me/", UserAPIView.as_view(), name="user-detail"),
    path('auth/google/', GoogleLogin.as_view(), name='google_login'),
    path('auth/secure/', Secure_api.as_view(), name='secure'),
    path('auth/ms/', MSLogin.as_view(), name='ms'),

]



from allauth.socialaccount.models import SocialApp
from django.core.management.base import BaseCommand
from django.db import transaction
from os import environ



SocialappList = [
    {
        "provider": "google",
        "name": "Google",
        "client_id": environ["GOOGLE_CLIENT_ID"],
        "secret": environ["GOOGLE_SECRET"],
    },
    {
        "provider": "microsoft",
        "name": "Microsoft",
        "client_id": environ["MS_CLIENT_ID"],
        "secret": environ["MS_SECRET"],
        
    },
]



class Command(BaseCommand):
    """
    Initialize Social Apps
    """

    @transaction.atomic
    def _init_providers(self):
        self.stdout.write(self.style.NOTICE("===== Initate Social Application ====="))
        rowsadded=0
        rowsskipped=0
        for socialapp in SocialappList:
            if SocialApp.objects.filter(name=socialapp.get('name')).exists():
               #self.stdout.write(self.style.WARNING(organization.get('code'))+" Org already exists!")
               rowsskipped+=1
            else:
                rowsadded+=1
                newsocialapp = SocialApp.objects.create(**socialapp)
                newsocialapp.save()
        self.stdout.write(self.style.SUCCESS(str(rowsadded))+" rows inserted! and " + self.style.SUCCESS(str(rowsskipped))+" rows already inserted! ")

    def handle(self, *args, **options):
        self._init_providers()
